@@ -2,9 +2,14 @@
 # Copyright (c) 2004-2006 Divmod.
 # See LICENSE for details.
 
-from nevow._version import version
-__version_info__ = (version.major, version.minor, version.micro)
-__version__ = version.short()
+from nevow._version import get_versions
+__version__ = get_versions()["version"]
+__version_info__ = tuple(int(part) for part in __version__.split("-", 1)[0].split(".")[:3])
+del get_versions
+
+from twisted.python.versions import Version
+version = Version("nevow", *__version_info__)
+del Version
 
 import sys
 from twisted.python.components import registerAdapter
@@ -218,5 +223,5 @@ loadFlatteners(flatteners)
 
 __all__ = [
     'accessors', 'appserver', 'blocks', 'canvas', 'context', 'dirlist', 'entities', 'events', 'failure', 'guard', 'inevow',
-    'loaders', 'rend', 'scripts', 'stan', 'static', 'tags', 'test', 'testutil', 'url', 'util', 'vhost', 'flat'
+    'loaders', 'rend', 'scripts', 'stan', 'static', 'tags', 'test', 'testutil', 'url', 'util', 'vhost', 'flat', 'version',
 ]
