@@ -12,7 +12,7 @@ is kind of close.
 import re, types
 
 from nevow.inevow import IAthenaTransportable
-from nevow import rend, page, flat, tags
+from nevow import rend, page, _flat, tags
 
 class ParseError(ValueError):
     pass
@@ -235,6 +235,7 @@ class CycleError(Exception):
 
 _translation = dict([(o, u'\\x%02x' % (o,)) for o in range(0x20)])
 
+# Characters which cannot appear as literals in the output
 _translation.update({
     ord(u'\\'): u'\\\\',
     ord(u'"'): ur'\"',
@@ -243,6 +244,10 @@ _translation.update({
     ord(u'\n'): ur'\n',
     ord(u'\t'): ur'\t',
     ord(u'\r'): ur'\r',
+    # The next two are sneaky, see
+    # http://timelessrepo.com/json-isnt-a-javascript-subset
+    ord(u'\u2028'): u'\\u2028',
+    ord(u'\u2029'): u'\\u2029',
     })
 
 def stringEncode(s):
@@ -288,7 +293,9 @@ def _serialize(obj, w, seen):
     elif isinstance(obj, (rend.Fragment, page.Element)):
         wrapper = tags.div(xmlns="http://www.w3.org/1999/xhtml")
         w('"')
-        w(stringEncode(flat.flatten(wrapper[obj]).decode('utf-8')))
+        w(stringEncode(
+                "".join(_flat.flatten(None, wrapper[obj],
+                                      False, False)).decode('utf-8')))
         w('"')
     else:
         transportable = IAthenaTransportable(obj, None)
