@@ -3,6 +3,8 @@ Tests for L{nevow.testutil} -- a module of utilities for testing Nevow
 applications.
 """
 
+import sys
+
 from unittest import TestResult
 
 from twisted.python.filepath import FilePath
@@ -47,8 +49,9 @@ class TestFakeRequest(TestCase):
         Verify that L{FakeRequest.prePathURL} will turn the C{Host} header of
         the request into the netloc of the returned URL, if it's present.
         """
-        req = FakeRequest(currentSegments=['a', 'b'], uri='/a/b/c/')
-        req.setHeader('host', 'foo.bar')
+        req = FakeRequest(currentSegments=['a', 'b'],
+                          uri='/a/b/c/',
+                          headers={'host': 'foo.bar'})
         self.assertEqual(req.prePathURL(), 'http://foo.bar/a/b')
 
 
@@ -67,24 +70,23 @@ class TestFakeRequest(TestCase):
 
     def test_headers(self):
         """
-        Check that one can get headers from L{FakeRequest} after they
-        have been set with L{FakeRequest.setHeader}.
+        Check that setting a header with L{FakeRequest.setHeader} actually
+        places it in the headers dictionary.
         """
         host = 'divmod.com'
         req = FakeRequest()
         req.setHeader('host', host)
-        self.assertEqual(req.getHeader('host'), host)
+        self.assertEqual(req.headers['host'], host)
 
 
     def test_caseInsensitiveHeaders(self):
         """
-        L{FakeRequest.getHeader} will return the value of a header previously
-        set with L{Request.setHeader} even if the header names have differing
-        case.
+        L{FakeRequest.getHeader} will return the value of a header regardless
+        of casing.
         """
         host = 'example.com'
         request = FakeRequest()
-        request.setHeader('HoSt', host)
+        request.received_headers['host'] = host
         self.assertEqual(request.getHeader('hOsT'), host)
 
 
@@ -128,6 +130,30 @@ class TestFakeRequest(TestCase):
         self.assertEqual(req.code, BAD_REQUEST)
 
 
+    def test_headerSeparation(self):
+        """
+        Request headers and response headers are different things.
+
+        Test that they are handled separately.
+        """
+        req = FakeRequest()
+        req.setHeader('foo', 'bar')
+        self.assertNotIn('foo', req.received_headers)
+        self.assertEqual(req.getHeader('foo'), None)
+        req.received_headers['foo'] = 'bar'
+        self.assertEqual(req.getHeader('foo'), 'bar')
+
+
+
+    def test_path(self):
+        """
+        Test that the path attribute of a fake request is set.
+        """
+        req = FakeRequest(uri='/foo')
+        self.assertEqual(req.path, '/foo')
+
+
+
 class JavaScriptTests(TestCase):
     """
     Tests for the JavaScript UnitTest runner, L{JavaScriptTestCase}.
@@ -152,6 +178,9 @@ class JavaScriptTests(TestCase):
         self.case.createSource = lambda testMethod: "throw new TypeError();"
         self.case.run(result)
         self.assertEqual(len(result.errors), 1)
+        self.assertTrue(
+            result.errors[0][1].startswith(
+                'Exception: JavaScript interpreter had error exit: '))
 
 
     def test_signalledExit(self):
@@ -159,10 +188,35 @@ class JavaScriptTests(TestCase):
         An error should be reported if the JavaScript interpreter exits because
         it received a signal.
         """
+        segfault = FilePath(self.mktemp())
+        segfault.setContent("""\
+#!/usr/bin/python
+# Generate an unhandled SIGSEGV for this process immediately upon import.
+
+import os, signal
+os.kill(os.getpid(), signal.SIGSEGV)
+""")
+
         def stubFinder():
-            return FilePath(__file__).sibling('segfault.py').path
+            return sys.executable
+        def stubScript(testModule):
+            return segfault.path
         self.case.findJavascriptInterpreter = stubFinder
-        self.case.createSource = lambda testMethod: ""
+        self.case.makeScript = stubScript
         result = TestResult()
         self.case.run(result)
         self.assertEqual(len(result.errors), 1)
+        self.assertEquals(
+            result.errors[0][1],
+            'Exception: JavaScript interpreter exited due to signal 11\n')
+
+
+    def test_missingJavaScriptClass(self):
+        """
+        If a JavaScript class required by the test code is unavailable, an
+        error is added to the result object by L{JavaScriptTestCase.run}.
+        """
+        result = TestResult()
+        self.case.testMethod = lambda: "Nevow.Test.NoSuchModule"
+        self.case.run(result)
+        self.assertEqual(len(result.errors), 1)
